/*
 * example sketch to list a directory and create/read/write a file
 * on an mmc, using the arduino wrapper function library for fatfs
 *
 * (c) 2010 david sirkin sirkin@stanford.edu & megan wachs wachs@stanford.edu
 */

/* import the avr's power library for the clock_prescale_set macro. */

#include <avr/power.h>

/* import the full fatfs library for access to functions & defines. */

#include <diskio.h>
#include <fatfs.h>
#include <ff.h>
#include <ffconf.h>
#include <integer.h>

#define NO_GRAPHICS
#define NO_BITMAP
#include <nokia_3310_lcd.h>

#define cs_pin        14           // chip select pin on the arduino
#define bytes_to_read (14*6)       // this is the max # of chars the
                                   // lcd can display
#define LCD_PWR       11
#define LCD_DC        4
#define LCD_SCE       21
#define LCD_RESET     5

Nokia_3310_lcd lcd(LCD_PWR,
                 LCD_DC,
                 LCD_SCE,
                 LCD_RESET);

/*
 * initialize the processor speed, setup the fatfs sd card (or, mms)
 * filesystem and initialize/clear the lcd.
 */
void setup() {
  // the avr atmega32u4 processor is only rated to 8mhz (rather than
  // 16mhz) when powered at 3.3v (rather than 5v).
  
  clock_prescale_set(clock_div_2); // fosc 8mhz as teensy is at 3.3v
  FatFs.begin(cs_pin);             // initialize FatFS library calls
  lcd.init();                      // initialize the graphical lcd
  lcd.clear();
}

/* read in buffer 'bytes' of 'bytes_to_read' size from the open file
 * object (test.txt) and send the output to the graphical lcd.
 */
void file_read_and_print_to_GLCD (void) {
  char bytes[bytes_to_read];       // buffer of bytes read from card
  unsigned int bytes_read;         // number of bytes read each call
  unsigned long bytes_count = 0;   // running count of bytes read
  
  lcd.clear();
  FatFs.lseek(0);                  // set file r/w pointer back to 0
  
  FatFs.read(bytes, bytes_to_read, &bytes_read);
  bytes_count += bytes_read;
  
  for (int i = 0; i < bytes_read; i++){
    if (bytes[i] == '\n') break;   // only print out the first line
    lcd.writeChar(bytes[i], MODE_NORMAL);
  }
}

/*
 * open file test.txt, read the contents, then print that out to the
 * graphical lcd.
 */
void loop() {
  // visit: http://elm-chan.org/fsw/ff/00index_e.html for filesystem
  // api documentation. note that most function names in our wrapper
  // library do not carry forward the original module's 'f_' prefix.
  
  // visit: http://elm-chan.org/fsw/ff/en/open.html for detail about
  // the file access (fa_read) and result (fr_ok) codes used here.

  if(FatFs.open("test.txt", FA_READ) == FR_OK) {
    file_read_and_print_to_GLCD();
  }
  else {
    lcd.writeString(0, 2, "ERROR", MODE_INVERSE);
  }
  while(1);
}
