/*
 * Arduino Wrapper Function Library for FatFs
 * (c) 2010, David Sirkin sirkin@stanford.edu
 *
 * FatFS by ChaN:
 * http://elm-chan.org/fsw/ff/00index_e.html
 */
 
#ifndef FFS_h
#define FFS_h

extern "C"
{
#include "ff.h"
}

#include "WProgram.h"

class FFS
{
  private:
  	static FATFS fatfs_obj;
  	static DIR dir_obj;
  	static FIL fil_obj;
  	static FILINFO fno;
  	
  	static unsigned char CS;
  	
  	static void CS_LOW (void);
  	static void CS_HIGH (void);
  public:
    FFS();

	/*--------------------------------------------------------------*/
	/* Functions for Local Disk Control                             */
	/*--------------------------------------------------------------*/

	FRESULT begin (unsigned char);
    DSTATUS disk_init (unsigned char);
    void io_init (void);
    void timerproc (void);
    
	/*--------------------------------------------------------------*/
	/* Functions in the FatFs Module Application Interface          */
	/*--------------------------------------------------------------*/
    /* Overloaded methods use working copy of FATFS, DIR or FIL     */
    
	FRESULT mount (unsigned char, FATFS*);				/* Mount/Unmount a logical drive */
	FRESULT open (FIL*, const XCHAR*, unsigned char);	/* Open or create a file */
	FRESULT open (const XCHAR*, unsigned char);
	FRESULT read (FIL*, void*, UINT, UINT*);			/* Read data from a file */
	FRESULT read (void*, UINT, UINT*);
	FRESULT write (FIL*, const void*, UINT, UINT*);		/* Write data to a file */
	FRESULT write (const void*, UINT, UINT*);
	FRESULT lseek (FIL*, DWORD);						/* Move file pointer of a file object */
	FRESULT lseek (DWORD);
	FRESULT close (FIL*);								/* Close an open file object */
	FRESULT close (void);
	FRESULT opendir (DIR*, const XCHAR*);				/* Open an existing directory */
	FRESULT opendir (const XCHAR*);
	FRESULT opendir (void);
	FRESULT readdir (DIR*, FILINFO*);					/* Read a directory item */
	FRESULT readdir (FILINFO*);
	FRESULT readdir (void);
	FRESULT stat (const XCHAR*, FILINFO*);				/* Get file status */
	FRESULT getfree (const XCHAR*, DWORD*, FATFS**);	/* Get number of free clusters on the drive */
	FRESULT truncate (FIL*);							/* Truncate file */
	FRESULT truncate (void);
	FRESULT sync (FIL*);								/* Flush cached data of a writing file */
	FRESULT sync (void);
	FRESULT unlink (const XCHAR*);						/* Delete an existing file or directory */
	FRESULT	mkdir (const XCHAR*);						/* Create a new directory */
	FRESULT chmod (const XCHAR*, unsigned char, unsigned char);	/* Change attriburte of the file/dir */
	FRESULT utime (const XCHAR*, const FILINFO*);		/* Change timestamp of the file/dir */
	FRESULT rename (const XCHAR*, const XCHAR*);		/* Rename/Move a file or directory */
	FRESULT forward (FIL*, UINT(*)(const unsigned char*,UINT), UINT, UINT*);	/* Forward data to the stream */
	FRESULT mkfs (unsigned char, unsigned char, WORD);	/* Create a file system on the drive */
	FRESULT chdir (const XCHAR*);						/* Change current directory */
	FRESULT chdrive (unsigned char);					/* Change current drive */

#if _USE_STRFUNC
	int fputc (int, FIL*);								/* Put a character to the file */
	int fputc (int);
	int puts (const char*, FIL*);						/* Put a string to the file */
	int puts (const char*);
	int printf (FIL*, const char*, ...);				/* Put a formatted string to the file */
	int printf (const char*, ...);
	char* gets (char*, int, FIL*);						/* Get a string from the file */
	char* gets (char*, int);
#define f_eof(fp) (((fp)->fptr == (fp)->fsize) ? 1 : 0)
#define f_error(fp) (((fp)->flag & FA__ERROR) ? 1 : 0)
#ifndef EOF
#define EOF -1
#endif /* EOF */
#endif /* _USE_STRFUNC */
};

extern FFS FatFs;

#endif

